package nc.impl.bm.bmdata;

import com.alibaba.fastjson.JSONObject;
import nc.bs.dao.BaseDAO;
import nc.bs.framework.common.NCLocator;
import nc.bs.framework.common.RuntimeEnv;
import nc.bs.logging.Logger;
import nc.itf.hr.bm.bmdata.IBmToU9LogService;
import nc.itf.hr.bm.bmdata.IBmdataSyncService;
import nc.itf.hr.wa.erp.IWaToU9LogService;
import nc.jdbc.framework.processor.ColumnListProcessor;
import nc.jdbc.framework.processor.ColumnProcessor;
import nc.jdbc.framework.processor.MapListProcessor;
import nc.utils.AnxjHttpClient;
import nc.utils.U9Interface;
import nc.vo.bm.sync.BmDataToU9HVO;
import nc.vo.bm.sync.BmToU9LogHVO;
import nc.vo.pub.BusinessException;
import nc.vo.pub.lang.UFDateTime;
import nc.vo.wa.erp.WaToU9LogHVO;
import nccloud.openapi.hrwa.vo.WaDataToU9BVO;
import nccloud.openapi.hrwa.vo.WaDataToU9HVO;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

public class BmdataSyncServiceImpl implements IBmdataSyncService {

    public BaseDAO dao;
    private SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    public BaseDAO getBaseDAO() {
        if (dao == null) {
            dao = new BaseDAO();
        }
        return dao;
    }

    private static Properties props = new Properties();

    private final static String ERROR_STATUS = "2";
    private final static String SUCCESS_STATUS = "1";

    static {
        if (props.size() <= 0) {
            String propertyUrl = RuntimeEnv.getInstance().getNCHome() + File.separator + "resources/U9url.properties";
            try {
                FileInputStream in = new FileInputStream(propertyUrl);
                props.load(in);
            } catch (IOException e) {
                e.printStackTrace();
                throw new RuntimeException(propertyUrl + "===U9url.properties not found");
            }
        }
    }


    @Override
    public List<String> pushToU9(String pk_group, String pk_org, String pk_bm_class, String cyear, String cperiod, String push_type, String provision) throws BusinessException {
        List<String> res = new ArrayList<>();

        List<WaDataToU9BVO> reportDataByParam = getReportDataByParam(pk_org, pk_bm_class, cyear, cperiod);
        if (reportDataByParam != null && reportDataByParam.size() > 0) {
            Map<String, List<WaDataToU9BVO>> erporgcode = reportDataByParam.stream().collect(Collectors.groupingBy(WaDataToU9BVO::getErporgcode)); // Ҫݲֵ֯ΪǷ֯
            for (Map.Entry<String, List<WaDataToU9BVO>> entry : erporgcode.entrySet()) {
                String erporgCode = entry.getKey();
                List<WaDataToU9BVO> erporgData = entry.getValue();
                String erporg = erporgData.get(0).getErporg();
                List<String> name = checkPush(pk_org, pk_bm_class, cyear, cperiod, erporg);

                if (name != null && name.size() > 0) {
                    res.addAll(name);
                } else {
                    String remark = (String) getBaseDAO()
                            .executeQuery(" SELECT o.NAME ||'-'|| c.name FROM bm_bmclass c JOIN org_orgs o on o.PK_ORG = c.PK_ORG  WHERE c.PK_BM_CLASS = '" + pk_bm_class + "' ", new ColumnProcessor());
                    WaDataToU9HVO waDataToU9HVO = new WaDataToU9HVO();
                    waDataToU9HVO.setErporgcode(erporgCode);
                    waDataToU9HVO.setCperiod(cperiod);
                    waDataToU9HVO.setCyear(cyear);
                    waDataToU9HVO.setHr_pk(pk_bm_class);
                    waDataToU9HVO.setHr_item(erporgData);
                    waDataToU9HVO.setPush_type(push_type);
                    waDataToU9HVO.setErporg(erporg);
                    waDataToU9HVO.setHr_type("bm");
                    waDataToU9HVO.setProvision(provision);
                    waDataToU9HVO.setRemark(remark);
                    BmToU9LogHVO initLogVO = getInitLogVO(pk_org, pk_bm_class, cyear, cperiod, erporg, erporgCode);
                    String paramStr = JSONObject.toJSONString(waDataToU9HVO);
                    initLogVO.setParam(paramStr);
                    try {
                        String u9Res = U9Interface.syncWaToU9(paramStr);
                        initLogVO.setContent(u9Res);
                        JSONObject u9ResJson = JSONObject.parseObject(u9Res);
                        JSONObject data = u9ResJson.getJSONObject("Data");
                        Boolean isSucess = data.getBoolean("IsSucess");
                        if (isSucess) {
                            // ɹ ¼־
                            getBaseDAO().executeUpdate("update bm_u9_log_h set is_flag = 'N'  where is_flag = 'Y' and  pk_erporg ='" + initLogVO.getPk_erporg() + "' and cperiod = '"
                                    + initLogVO.getCperiod() + "' and cyear = '" + initLogVO.getCyear() + "' and pk_bm_class ='" + initLogVO.getPk_bm_class() + "' ");
                            initLogVO.setR_status(SUCCESS_STATUS);
                            initLogVO.setIs_flag("Y");
                        } else {
                            initLogVO.setR_status(ERROR_STATUS);
                            Logger.error("籣ͬʧܲ" + paramStr);
                            Logger.error("籣ͬʧܷؽ" + res);
                            throw new BusinessException("ͬU9ʧ" + data.toString());
                        }
                    } catch (Exception e) {
                        Logger.error("籣ͬʧܲ" + paramStr);
                        initLogVO.setR_status(ERROR_STATUS);
                        initLogVO.setContent(e.getMessage());
                        throw new BusinessException("ͬU9ʧ" + e.getMessage());
                    } finally {
                        NCLocator.getInstance().lookup(IBmToU9LogService.class).insertLog_RequiresNew(initLogVO);
                    }
                }
            }

        } else {
            throw new BusinessException("ΪգǷȱʧ");
        }
        return res;
    }


    BmToU9LogHVO getInitLogVO(String pk_org, String pk_bm_class, String cyear, String cperiod, String pk_erporg, String erporgCode) {
        BmToU9LogHVO bmToU9LogHVO = new BmToU9LogHVO();
        bmToU9LogHVO.setCreation_time(new UFDateTime());
        bmToU9LogHVO.setPk_bm_class(pk_bm_class);
        bmToU9LogHVO.setPk_org(pk_org);
        bmToU9LogHVO.setCyear(cyear);
        bmToU9LogHVO.setCperiod(cperiod);
        bmToU9LogHVO.setSend_time(sdf.format(new Date()));
        bmToU9LogHVO.setPk_erporg(pk_erporg);
        bmToU9LogHVO.setErporgCode(erporgCode);
        return bmToU9LogHVO;
    }

    public List<String> checkPush(String pk_org, String pk_bm_class, String cyear, String cperiod, String erporg) throws BusinessException {
        /**
         * Уܷ u9 Ŀǰ һ  ظƣU9 ״̬ڱ̬δ̣  ݸ޸ģ Ҳͳɹ
         * ̣U9ᱨ   ꣬ͨ  ص޸־״̬    Чʧܺ Լ
         */
        String whereSql = " SELECT DISTINCT doc.name from bm_u9_log_h log join bd_defdoc doc on log.pk_erporg = doc.pk_defdoc where" +
                " log.pk_org = '" + pk_org + "' and log.pk_bm_class = '" + pk_bm_class + "' and log.cyear = '"
                + cyear + "' and log.cperiod = '" + cperiod + "' and log.r_status = '200' and log.pk_erporg = '" + erporg + "' ";
        List<String> waToU9LogHVOS = (List<String>) getBaseDAO().executeQuery(whereSql, new ColumnListProcessor());
        return waToU9LogHVOS;
    }

    private List<WaDataToU9BVO> getReportDataByParam(String pk_org, String pk_bm_class, String cyear, String cperiod) throws BusinessException {
        // 1.̬ȡֵĿ  Զ嵵 ʱ޸ֶ
        Map<String, String> allocateItem = getAllocateItem();
        StringBuffer sb = new StringBuffer();
        Set<Map.Entry<String, String>> entries = allocateItem.entrySet();
        checkData(pk_org, pk_bm_class, cyear, cperiod, entries);

        for (Map.Entry<String, String> entry : entries) {
            sb.append(", sum(nvl(bd." + entry.getValue() + ",0)) as " + entry.getKey());
        }

        String sql = " SELECT  bd.glbdef1 erporg, erporg.code erporgcode, erpdept.code erpdeptcode,ft.CODE fee_type,count(1) as psnsum  " + sb.toString() +
                " FROM bm_data bd " +
                " join (SELECT  b.code,b.name FROM bd_defdoc b JOIN  bd_defdoclist h ON h.pk_defdoclist = b.pk_defdoclist " +
                " WHERE h.code = 'HRZ04' AND b.enablestate = 2 AND nvl(b.DATATYPE,1) <> 0)  ft on ft.name = bd.C_1 " +  //   ˵Ϊ籣ֶͲԶֶ Դ  c_1
                " JOIN (SELECT  b.pk_defdoc,b.code,b.name FROM bd_defdoc b JOIN  bd_defdoclist h ON h.pk_defdoclist = b.pk_defdoclist  " +
                " WHERE h.code = 'HRZ07' AND b.enablestate = 2 AND nvl(b.DATATYPE,1) <> 0) erporg ON erporg.pk_defdoc = bd.glbdef1  " +
                " JOIN (SELECT  b.pk_defdoc,b.code,b.name FROM bd_defdoc b JOIN  bd_defdoclist h ON h.pk_defdoclist = b.pk_defdoclist  " +
                " WHERE h.code = 'HRZ06' AND b.enablestate = 2 AND nvl(b.DATATYPE,1) <> 0) erpdept ON erpdept.pk_defdoc = bd.glbdef2   " +
                " WHERE bd.pk_bm_class  = '" + pk_bm_class + "' and bd.cyear = '" + cyear + "' and bd.cperiod = '" + cperiod + "' and bd.checkflag = 'Y' and bd.accountstate = 0 AND bd.pk_org = '" + pk_org + "' " +
                " GROUP BY  bd.glbdef1, erporg.code,erpdept.code,ft.code ";
        List<Map<String, Object>> docs = (List<Map<String, Object>>) getBaseDAO().executeQuery(sql, new MapListProcessor());
        List<WaDataToU9BVO> waDataToU9BVOS = new ArrayList<>();
        if (docs != null && docs.size() > 0) {
            docs.forEach(doc -> {
                for (Map.Entry<String, String> entry : entries) {
                    BigDecimal amount = new BigDecimal(ObjectUtils.toString(doc.get(entry.getKey()), "0"));
                    if (BigDecimal.ZERO.compareTo(amount) == 0) continue; // 20231010 dzz ÷˵ûǮĲ
                    WaDataToU9BVO waDataToU9BVO = new WaDataToU9BVO();
                    waDataToU9BVO.setErporgcode((String) doc.get("erporgcode"));
                    waDataToU9BVO.setErporg((String) doc.get("erporg"));
                    waDataToU9BVO.setErpdeptcode((String) doc.get("erpdeptcode"));
                    waDataToU9BVO.setFee_type((String) doc.get("fee_type"));
                    waDataToU9BVO.setPsnsum((Integer) doc.get("psnsum"));
                    waDataToU9BVO.setXzxmbm(entry.getKey());
                    waDataToU9BVO.setAmount(amount);
                    waDataToU9BVOS.add(waDataToU9BVO);
                }
            });
        }


        return waDataToU9BVOS;

    }

    // ȡ籣ط̯Ŀ
    private Map<String, String> getAllocateItem() throws BusinessException {
        String sql = " SELECT  b.code,b.name,b.memo FROM bd_defdoc b JOIN  bd_defdoclist h ON h.pk_defdoclist = b.pk_defdoclist " +
                " WHERE h.code = 'HRZ08' AND b.enablestate = 2 AND nvl(b.DATATYPE,1) <> 0 AND b.code like 'sb%'";

        List<Map<String, Object>> doc = (List<Map<String, Object>>) getBaseDAO().executeQuery(sql, new MapListProcessor());
        if (doc != null && doc.size() > 0) {
            Map<String, String> map = doc.stream().collect(Collectors.toMap(e -> (String) e.get("code"), e -> (String) e.get("memo")));
            return map;
        } else {
            throw new BusinessException("ѯ̯ĿHRZ08籣Ϊ,");
        }
    }

    /**
     * 20231019
     *
     * @param pk_org
     * @param pk_bm_class
     * @param cyear
     * @param cperiod
     */
    private void checkData(String pk_org, String pk_bm_class, String cyear, String cperiod, Set<Map.Entry<String, String>> entries) throws BusinessException {
        String sql = " SELECT psn.name,bd.checkflag, bd.glbdef1 erporg, erporg.code erporgcode, erpdept.code erpdeptcode,ft.CODE fee_type" +
                " FROM bm_data bd " +
                "join bd_psndoc psn on psn.pk_psndoc = bd.pk_psndoc " +
                "left join (SELECT  b.code,b.name FROM bd_defdoc b JOIN  bd_defdoclist h ON h.pk_defdoclist = b.pk_defdoclist " +
                " WHERE h.code = 'HRZ04' AND b.enablestate = 2 AND nvl(b.DATATYPE,1) <> 0)  ft on ft.name = bd.C_1 " +  //   ˵Ϊ籣ֶͲԶֶ Դ  c_1
                "left JOIN (SELECT  b.pk_defdoc,b.code,b.name FROM bd_defdoc b JOIN  bd_defdoclist h ON h.pk_defdoclist = b.pk_defdoclist  " +
                " WHERE h.code = 'HRZ07' AND b.enablestate = 2 AND nvl(b.DATATYPE,1) <> 0) erporg ON erporg.pk_defdoc = bd.glbdef1  " +
                "left JOIN (SELECT  b.pk_defdoc,b.code,b.name FROM bd_defdoc b JOIN  bd_defdoclist h ON h.pk_defdoclist = b.pk_defdoclist  " +
                " WHERE h.code = 'HRZ06' AND b.enablestate = 2 AND nvl(b.DATATYPE,1) <> 0) erpdept ON erpdept.pk_defdoc = bd.glbdef2   " +
                " WHERE bd.pk_bm_class  = '" + pk_bm_class + "' and bd.cyear = '" + cyear + "' and bd.cperiod = '" + cperiod + "'  and bd.accountstate = 0 AND bd.pk_org = '" + pk_org + "' ";
        List<Map<String, Object>> data = (List<Map<String, Object>>) getBaseDAO().executeQuery(sql, new MapListProcessor());
        if (data != null && data.size() > 0) {
            List<String> checkflagArray = new ArrayList<>();
            List<String> fee_typeArray = new ArrayList<>();
            List<String> erporgcodeArray = new ArrayList<>();
            List<String> erpdeptcodeArray = new ArrayList<>();
            for (Map<String, Object> datum : data) {
                String checkflag = (String) datum.get("checkflag"); // 
                String fee_type = (String) datum.get("fee_type"); // 
                String erporgcode = (String) datum.get("erporgcode"); // ֯
                String erpdeptcode = (String) datum.get("erpdeptcode"); // 
                String name = (String) datum.get("name"); // 
                if (!"Y".equals(checkflag)) checkflagArray.add(name);
                if (StringUtils.isBlank(fee_type)) fee_typeArray.add(name);
                if (StringUtils.isBlank(erporgcode)) erporgcodeArray.add(name);
                if (StringUtils.isBlank(erpdeptcode)) erpdeptcodeArray.add(name);
            }
            StringBuffer sb = new StringBuffer();
            if (checkflagArray.size() > 0) sb.append("δԱ" + String.join(",", checkflagArray) + "");
            if (fee_typeArray.size() > 0) sb.append("ΪԱ" + String.join(",", fee_typeArray) + "");
            if (erporgcodeArray.size() > 0) sb.append("֯ΪԱ" + String.join(",", erporgcodeArray) + "");
            if (erpdeptcodeArray.size() > 0) sb.append("ΪԱ" + String.join(",", erpdeptcodeArray) + "");
            if (sb.length() > 0) {
                throw new BusinessException("U9ʧܣ" + sb.toString());
            }
        }
    }
}
